{% load i18n %}
<form action="" method="post" class="ff subscribe">{% csrf_token %}
    {{ form.non_field_errors }}

    {% if form.subject.errors %}
        <ol>
            {% for error in form.receiver.errors %}
                <li><strong>{{ error|escape }}</strong></li>
            {% endfor %}
        </ol>
    {% endif %}

    {{ form.receiver.errors }}
    {{ form.receiver }}
    <input type="submit" value="{% trans "Subscribe to the newsletter" %}" class="ff-btn btn-primary">
</form>
<div class="close">
    <a href=""><i class="sprite-flash-close">{% trans "Close" %}</i></a>
</div>
