<h1>{{ object.name }}</h1>
{% if object.headline %}<p>{{ object.headline }}</p>{% endif %}
<p>Published: {{ object.published_at|date }}</p>

<ul>
    {% for item in items %}
        <li>{{ item.description }}</li>
    {% endfor %}
</ul>
<a href="[[UNSUB_LINK_EN]]"></a>
