{% if previous_object %}
    <p>Previous : <a href="{% url newsletter_detail previous_object.pk %}">{{ previous_object.name }}</a></p>
{% endif %}
{% if next_object %}
    <p>Next : <a href="{% url newsletter_detail next_object.pk %}">{{ next_object.name }}</a></p>
{% endif %}

{% if messages %}
    <ul class="messages">
        {% for message in messages %}
            <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message }}</li>
        {% endfor %}
    </ul>
{% endif %}


<form action="" method="post">{% csrf_token %}
    {{ form.non_field_errors }}

    {% if form.subject.errors %}
        <ol>
            {% for error in form.receiver.errors %}
                <li><strong>{{ error|escape }}</strong></li>
            {% endfor %}
        </ol>
    {% endif %}

    <div class="fieldWrapper">
        {{ form.receiver.errors }}
        <label for="id_receiver">Your e-mail :</label>
        {{ form.receiver }}
    </div>
    <p><input type="submit" value="Submit"/></p>
</form>


<iframe src="{% url "newsletter_raw_detail" newsletter.pk %}" width="100%" height="600px" style="border:1px solid #DCDCDB"></iframe>
