{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}
{% block object-tools-items %}
    <li>
        <a href="{% url 'admin:send_newsletter' original.pk %}" class="historylink" onclick="return confirm('{% blocktrans %}Do you really want to send this newsletter?{% endblocktrans %}');">Send this newsletter</a>
    </li>
    <li>
        <a href="{% url opts|admin_urlname:'history' original.pk|admin_urlquote %}" class="historylink">{% trans "History" %}</a>
    </li>
    {% if has_absolute_url %}
        <li>
            <a href="{% url 'admin:view_on_site' content_type_id original.pk %}" class="viewsitelink">{% trans "View on site" %}</a>
        </li>
    {% endif%}
{% endblock %}