{% load i18n %}
{% if newsletter_list %}
<p>{% trans "Thank you. You're now unsubscribed from this newsletter." %}</p>
{% else %}
<p>{% trans "Thank you. You're now unsubscribed from all our newsletters." %}</p>
{% endif %}